package com.xhkjedu.smarking.controller.exam;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.xhkjedu.smarking.mapper.system.UserMapper;
import com.xhkjedu.smarking.model.exam.MsExam;
import com.xhkjedu.smarking.service.exam.MsExamService;
import com.xhkjedu.smarking.service.notice.NoticeService;
import com.xhkjedu.smarking.vo.exam.EBaseVo;
import com.xhkjedu.smarking.vo.exam.ExamVo;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.utils.PageUtil;
import com.xhkjedu.vo.PageResult;
import com.xhkjedu.vo.ResultVo;
import com.xhkjedu.vo.system.UserVo;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;

/**
 * @Description 阅卷考试表相关操作
 * @Author auto
 * @Date 2024-10-15
 */
@RestController
@RequestMapping("/msExam")
public class MsExamController {
    @Resource
    private MsExamService msExamService;
    @Resource
    private NoticeService noticeService;
    @Resource
    private UserMapper userMapper;

    /**
     * @Description 新增
     * @Param [msExam]
     * @Author auto
     * @Date 2024-10-15
     * @return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/save")
    public ResultVo save(@RequestBody MsExam msExam) {
        N_Utils.validation(new Object[]{
                msExam.getExamname(), "考试名称", 2
                , msExam.getExamdate(), "考试日期", 2
                , msExam.getGradeid(), "考试年级", 1
                , msExam.getExamtype(), "考试类型", 1
                , msExam.getExammode(), "阅卷模式", 1
                , msExam.getSubjects(), "考试科目", 3
                , msExam.getCreateid(), "创建人id", 1
                , msExam.getSchoolid(), "学校id", 1
        });
        msExamService.add(msExam);
        return new ResultVo(0, "保存成功！");
    }

    /**
     * @Description 列表
     * @Param [msExam]
     * @Author auto
     * @Date 2024-10-15
     * @return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/list")
    public ResultVo queryList(@RequestBody MsExam msExam) {
        Integer page = msExam.getPage();
        Integer size = msExam.getPageSize();
        N_Utils.validation(new Object[]{page, "显示页码", 1, size, "显示条数", 1});
        PageHelper.startPage(page, size);
        List<Map> list = msExamService.queryList(msExam);
        PageResult pageResult = PageUtil.getPageResult(new PageInfo<>(list));
        return new ResultVo(0, "查询成功！", pageResult);
    }

    /**
     * @Description 发布考试
     * @Date 2024/10/17 17:57
     * @Author YWX
     * @Param [vo]
     * @Return com.xhkjedu.vo.ResultVo
     **/
    @PostMapping("/fb")
    public ResultVo postExam(@RequestBody ExamVo vo) {
        Integer notice = vo.getNotice();//是否发送通知1发送2不发送
        Integer msg = vo.getMsg();//是否发送短信1发送2不发送
        String examcomm = vo.getExamcomm();
        Integer createid = vo.getHandleid();
        Integer examid = vo.getExamid();
        N_Utils.validation(new Object[]{examid, "考试id", 1, examcomm, "考试公告", 2, createid, "操作人id", 1});
        EBaseVo exam = msExamService.getExamStateById(examid);//根据考试id获取考试状态
        String examname = exam.getExamname();
        Integer schoolid = exam.getSchoolid();
        if (exam.getExammode() == 1 && " :00".equals(exam.getStrtime())) return new ResultVo(1, "还有考试试卷未设置时间");
        if (exam.getWwcnum() != 0) return new ResultVo(1, "还有考试试卷未设置完成");
        if (exam.getAnswered() == 0) return new ResultVo(1, "还有考试试卷未设置正确答案");
        if (exam.getExamstate() != 0) return new ResultVo(0, "发布成功");

        List<UserVo> students = userMapper.listStuByExamId(examid);//获取考生
        Integer code = msExamService.updateExamStateStart(examid, examcomm, students);//发布考试
        if (code == 1) return new ResultVo(1, "发布考试失败");

        if (notice == 2 && msg == 2) return new ResultVo(0, "发布成功");

        vo.setSchoolid(schoolid);
        vo.setHandleid(createid);
        vo.setExamname(examname);
        vo.setGradeid(exam.getGradeid());
        noticeService.batchSave2(vo, students);
        return new ResultVo(0, "保存成功！");
    }
}
