package com.xhkjedu.smarking.service.paper;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.exception.MissingParametersException;
import com.xhkjedu.smarking.mapper.exam.MsSubjectMapper;
import com.xhkjedu.smarking.mapper.paper.*;
import com.xhkjedu.smarking.mapper.system.UserBasketMapper;
import com.xhkjedu.smarking.model.paper.*;
import com.xhkjedu.smarking.utils.ConvertUtil;
import com.xhkjedu.smarking.vo.paper.*;
import com.xhkjedu.utils.N_Utils;
import com.xhkjedu.vo.ResultVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description 阅卷试卷题型表 服务实现类
 * @Author auto
 * @Date 2024-10-15
 */
@Service
public class MsPaperQtypeService {
    @Resource
    private MsPaperQtypeMapper msPaperQtypeMapper;
    @Resource
    private MsPaperAnalyzeMapper msPaperAnalyzeMapper;
    @Resource
    private MsPaperMapper msPaperMapper;
    @Resource
    private MsPaperQtypeQuestionMapper msPaperQtypeQuestionMapper;
    @Resource
    private MsPaperFileMapper msPaperFileMapper;
    @Resource
    private MsPaperQtypeQuestionPointMapper msPaperQtypeQuestionPointMapper;
    @Resource
    private MsSubjectMapper msSubjectMapper;
    @Resource
    private UserBasketMapper userBasketMapper;
    @Autowired
    private ConvertUtil convertUtil;

    /**
     * @Description 新增
     * @Param [msPaperQtype]
     * @Author auto
     * @Date 2024-10-15
     * @return void
     **/
    @Transactional(rollbackFor = Exception.class)
    public ResultVo savePaperQtype(MsPaper paper,Integer pstate){
        Integer mpid = paper.getMpid();

        //根据试卷ID获取考试信息
        PExamSubjectVo exam = msPaperMapper.getExamSubjectForPaper(mpid);
        if(exam.getExammode() < 3 && exam.getExamstate() > 2){
            throw new MissingParametersException("考试已结束禁止操作");
        }

        if(exam.getExammode() == 3 && exam.getExamstate() > 0){
            throw new MissingParametersException("考试已开始禁止操作");
        }

         //先删除原有题型、分析
        msPaperQtypeMapper.deleteByMpId(mpid);
        msPaperAnalyzeMapper.deleteByMpId(mpid);
        //根据考试类型（线上和线下），是否删除文件
        if(paper.getPtype() == 2){
            msPaperFileMapper.deleteByMpid(mpid);
        }

        List<MsPaperQtype> qtypeList = paper.getQtypes();
        int timestamp = N_Utils.getSecondTimestamp();
        paper.setCreatetime(timestamp);
        setPaperBaseDetail(paper,qtypeList);
        msPaperMapper.updatePaperInfo(paper);
        //保存题型试题信息
        savePaperQtype(paper,qtypeList);

        List<String> imgs = new ArrayList<>();
        //保存试卷分析
        if(paper.getPtype() == 1){
            savePaperAnalyzeForQuestion(paper);
        }else{
            List<MsPaperFile> pfiles = paper.getPfiles();//附件
            if(paper.getHasfile() == 1){
                setPaperFiles(paper,pfiles,imgs);
                msPaperFileMapper.insertList(pfiles);
            }
            savePaperAnalyzeForFj(paper);
        }

        //更改科目状态
        msSubjectMapper.updatePstateByMpid(mpid, pstate);
        if(paper.getPtype() == 1){
            userBasketMapper.deleteByUseridAndSubjectid(paper.getCreateid(),exam.getSubjectid());
        }

        if(exam.getExammode() < 3){
            //如果是线下考试更改了试卷结构，则清空后续所有操作
            msPaperMapper.deleteTemplageBlockByMpid(mpid);
        }

        if(N_Utils.isListNotEmpty(imgs) && imgs.size()>1){
            //说明上传的附件文件为图片，并且图片为多张，调用mq合并
            convertUtil.imgToPdf(imgs,mpid);
        }

        return new ResultVo(0,"保存成功",paper.getMpid());
    }

    //设置试卷基本信息
    private void setPaperBaseDetail(MsPaper paper,List<MsPaperQtype> qtypelist){
        if(paper.getPtype() == 2 && paper.getPfiles()!=null && paper.getPfiles().size()>0){
            paper.setHasfile(1);
        }else {
            paper.setHasfile(0);
        }
        double pscore = qtypelist.stream().collect(Collectors.summingDouble(MsPaperQtype:: getMptscore));
        int pnum = qtypelist.stream().collect(Collectors.summingInt(MsPaperQtype:: getMptnum));
        paper.setPnum(pnum);
        paper.setPscore(pscore);
        paper.setHandletime(paper.getCreatetime());
        paper.setHandleid(paper.getCreateid());
        if(paper.getPtype() == 1){
            paper.setAnswered(1);//题库已设置正确答案
        }else{
            paper.setAnswered(0);
        }

        if(paper.getHearnum() == null){
            paper.setHearnum(0);
        }
    }

    //保存试卷题型试题信息
    private void savePaperQtype(MsPaper paper,List<MsPaperQtype> qtypelist){
        for(MsPaperQtype qtype : qtypelist){
            qtype.setMpid(paper.getMpid());
            qtype.setCreateid(paper.getHandleid());
            qtype.setCreatetime(paper.getHandletime());
            qtype.setMptname(N_Utils.strTrim(qtype.getMptname()));
        }
        msPaperQtypeMapper.insertList(qtypelist);//保存题型
        List<MsPaperQtypeQuestion> qtqs = new ArrayList<>();
        for(MsPaperQtype qtype : qtypelist){
            List<MsPaperQtypeQuestion> questions = qtype.getQuestions();
            qtype.setMptid(qtype.getId());

            for (int i = 0; i < questions.size(); i++) {
                MsPaperQtypeQuestion q=questions.get(i);
                q.setMpid(paper.getMpid());
                q.setMptid(qtype.getId());
                if(q.getOptionnum()==null){
                    q.setOptionnum(0);
                }
                if(q.getLoseoption() == null){
                    q.setLoseoption(0);
                }
                if(q.getScoreset() == null){
                    q.setScoreset(0);
                }

                qtqs.add(q);
            }
        }
        msPaperQtypeQuestionMapper.insertList(qtqs);
    }

    //保存试卷分析--题库
    public void savePaperAnalyzeForQuestion(MsPaper paper){
        //保存之前先清除之前的试卷分析
        msPaperAnalyzeMapper.deleteByMpId(paper.getMpid());
        //总体分析（主观题、客观题）
        List<MsPaperQtypeVo> qtypelist = msPaperQtypeMapper.listPaperQtypeQuestions(paper.getMpid());//试卷题型试题信息
        List<MsPaperQtypeQuestion> uplist = new ArrayList<>();//改试卷中客观题选项数量
        List<QuestionOrderVo> pointqlist = new ArrayList<>();//用于存放知识点，以及知识点下得分
        List<MsPaperQtypeQuestionVo> mquestions = new ArrayList<>();//存放单题和母题
        List<QuestionOrderVo> complexitylist = new ArrayList<>();//试题难易度存放试题
        List<QuestionPointVo> quesplist = new ArrayList<>();//单题、母题对应知识点

        //处理出试卷中所有试题、知识点
        List<Map> typelist = new ArrayList<>();//题型分析
        for(MsPaperQtypeVo qt : qtypelist){
            List<Integer> qorderlist = new ArrayList<>();
            List<QuestionOrderVo> qtypeqlist = new ArrayList<>();
            List<MsPaperQtypeQuestionVo> tqlist = qt.getQuestions();
            int hasmt = 0;
            for(MsPaperQtypeQuestionVo q : tqlist){
                int ctype = q.getCtype();
                //单题的pid设置为默认值，方便后续进行单题和母题处理
                if(q.getQlevel()==1){
                    q.setQuestionpid("000");
                    //判断试题是否是客观题
                    if(N_Utils.isObjectiveQuestion(ctype)){
                        //判断试题是否是客观题,暂时用mctype进行标识
                        q.setMctype(111);
                    }else{
                        q.setMctype(112);
                    }
                    //设置知识点
                    mquestions.add(q);
                    //获取试题知识点
                    List<Map> qpoints = q.getPoints();
                    //处理试题知识点占分
                    setPaperQuestionPointScore(pointqlist,quesplist,qpoints,q,null,null);
                    setPaperQuestionQtypeScore(qtypeqlist,q,null,null);//处理试题题型
                    setPaperQuestionComplexityScore(complexitylist,q,null,null);//处理试题难易度
                }
                if(q.getQlevel() == 3){
                    hasmt ++;
                }
                if(!qorderlist.contains(q.getQorder())){
                    qorderlist.add(q.getQorder());
                }
                //为了处理客观题中选项数量
                if(ctype == 1 || ctype ==2 || ctype==4 || ctype ==5 || ctype==6){
                    List<String> strlst = JSON.parseArray(q.getQoption(), String.class);
                    MsPaperQtypeQuestion eqq = new MsPaperQtypeQuestion();
                    eqq.setMptqid(q.getMptqid());
                    eqq.setOptionnum(strlst.size());
                    uplist.add(eqq);
                }
            }

            if(hasmt > 0){
                //说明该题型下有复合题,复合体进行知识点、题型下试题题号及id进行操作
                LinkedHashMap<String,List<MsPaperQtypeQuestionVo>> quesMap = tqlist.stream().collect(Collectors.groupingBy(MsPaperQtypeQuestionVo:: getQuestionpid, LinkedHashMap::new, Collectors.toList()));
                for(Map.Entry<String, List<MsPaperQtypeQuestionVo>> entry : quesMap.entrySet()){
                    if(!entry.getKey().equals("000")){
                        //说明该题为母题，
                        MsPaperQtypeQuestionVo mq = new MsPaperQtypeQuestionVo();
                        List<MsPaperQtypeQuestionVo> sonques = entry.getValue();
                        int k = 0;//如果k大于1说明有小题是主观题，则母题为主观题
                        double score = 0;
                        for(MsPaperQtypeQuestionVo sq : sonques){
                            if(!N_Utils.isObjectiveQuestion(sq.getCtype())){
                                k++;
                            }
                            score = N_Utils.getDoubleSum(score,sq.getScore());
                        }

                        MsPaperQtypeQuestionVo qobj = sonques.get(0);
                        mq.setQuestionid(qobj.getQuestionpid());
                        mq.setComplexity(qobj.getMcomplexity());
                        mq.setQorder(qobj.getQorder());
                        mq.setScore(score);
                        mq.setQtypename(qobj.getMqtypename());
                        if(k == 0){
                            mq.setMctype(111);
                        }else{
                            mq.setMctype(112);
                        }
                        mq.setQlevel(qobj.getMqlevel());
                        mq.setSonques(sonques);
                        //获取所有子题的试题题号
                        List<String> qns = sonques.stream().map(MsPaperQtypeQuestionVo::getQn).collect(Collectors.toList());
                        String[] qnstr = qns.stream().toArray(String[]::new);
                        //所有子题的试题eptqid
                        List<Integer> eptqids = sonques.stream().map(MsPaperQtypeQuestionVo::getMptqid).collect(Collectors.toList());
                        Integer[] eptqidstr = eptqids.stream().toArray(Integer[]::new);

                        //获取知识点
                        List<Map> qpoints = sonques.get(0).getPoints();
                        mq.setPoints(qpoints);
                        mquestions.add(mq);
                        setPaperQuestionPointScore(pointqlist,quesplist,qpoints,mq,qnstr,eptqids);
                        setPaperQuestionQtypeScore(qtypeqlist,mq,qnstr,eptqidstr);//处理试题题型
                        setPaperQuestionComplexityScore(complexitylist,mq,qnstr,eptqidstr);//处理试题难易度
                    }
                }
            }

            //试题题型分析数据
            double storerate = N_Utils.getDoubleDivideAndMulitiply(qt.getMptscore(), paper.getPscore());
            Map tmap = new TreeMap();
            tmap.put("qtid",qt.getMptid());
            tmap.put("qtname",qt.getMptname());
            tmap.put("num",qt.getMptnum());
            tmap.put("score",qt.getMptscore());
            tmap.put("srate",storerate);
            qtypeqlist = qtypeqlist.stream().sorted(Comparator.comparing(QuestionOrderVo::getOrder)).collect(Collectors.toList());
            tmap.put("ques",qtypeqlist);
            typelist.add(tmap);
        }

        //进行客观题选项处理
        if(N_Utils.isListNotEmpty(uplist)){
            msPaperQtypeQuestionMapper.updateBatchQuestionOptionnum(uplist);
        }

        //试题总体分布进行分析
        //客观题集合
        List<MsPaperQtypeQuestionVo> ktlst = mquestions.stream().filter(q->q.getMctype() == 111).collect(Collectors.toList());
        int ktnum = ktlst.size();
        double ktscore = ktlst.stream().collect(Collectors.summingDouble(MsPaperQtypeQuestionVo:: getScore));
        double ktnrate = N_Utils.getIntegerDivideAndMulitiply(ktnum, paper.getPnum());
        double ktsrate = N_Utils.getDoubleDivideAndMulitiply(ktscore, paper.getPscore());
        List<MsPaperQsVo> ranglist = new ArrayList<>();
        ranglist.add(new MsPaperQsVo("客观题",ktscore,ktnum,ktsrate,ktnrate));
        MsPaperQsVo zgq = new MsPaperQsVo();
        zgq.setQtname("主观题");
        zgq.setNum(paper.getPnum() - ktnum);
        zgq.setNrate(N_Utils.getDoubleReduce(100.0,ktnrate));
        zgq.setScore(N_Utils.getDoubleReduce(paper.getPscore(),ktscore));
        zgq.setSrate(N_Utils.getDoubleReduce(100.0,ktsrate));
        ranglist.add(zgq);
        Map ztmap = new TreeMap<>();
        ztmap.put("pnum", paper.getPnum());
        ztmap.put("pscore",paper.getPscore());
        ztmap.put("ranglist",ranglist);

        //难易度
        List<Map> clist = new ArrayList<>();
        if(N_Utils.isListNotEmpty(complexitylist)){
            Map<String,List<QuestionOrderVo>> complxmap =  complexitylist.stream().collect(Collectors.groupingBy(QuestionOrderVo:: getId, Collectors.toList()));
            for(Map.Entry<String, List<QuestionOrderVo>> entry : complxmap.entrySet()){
                List<QuestionOrderVo> pgblist = entry.getValue();
                Double score = pgblist.stream().mapToDouble(s -> s.getScore()).sum();
                score = N_Utils.formatDouble(score,1);
                Double srate = N_Utils.getDoubleDivideAndMulitiply(score,paper.getPscore());
                Map prate = new TreeMap();
                prate.put("level",Integer.parseInt(entry.getKey()));
                prate.put("score",score);
                prate.put("srate",srate);
                prate.put("ques",pgblist.stream().sorted(Comparator.comparing(QuestionOrderVo::getOrder)).collect(Collectors.toList()));
                clist.add(prate);
            }
        }

        //知识点分析
        List<Map> pointlist = new ArrayList<>();
        if(N_Utils.isListNotEmpty(pointqlist)){
            LinkedHashMap<String,List<QuestionOrderVo>> pointmap = pointqlist.stream().collect(Collectors.groupingBy(QuestionOrderVo::getId,LinkedHashMap::new,Collectors.toList()));
            //根据指示进行分组，
            for(Map.Entry<String, List<QuestionOrderVo>> entry : pointmap.entrySet()){
                List<QuestionOrderVo> pgblist = entry.getValue();
                //获取知识点的总分
                Double score = pgblist.stream().mapToDouble(s -> s.getScore()).sum();
                score = N_Utils.formatDouble(score,1);
                Double srate = N_Utils.getDoubleDivideAndMulitiply(score,paper.getPscore());
                Map prate = new LinkedHashMap();
                prate.put("pointid",entry.getKey());
                prate.put("pointname",pgblist.get(0).getName());
                prate.put("score",score);
                prate.put("srate",srate);
                prate.put("ques",pgblist.stream().sorted(Comparator.comparing(QuestionOrderVo::getOrder)).collect(Collectors.toList()));
                pointlist.add(prate);
            }
        }

        MsPaperAnalyze epa = new MsPaperAnalyze();
        epa.setMpid(paper.getMpid());
        epa.setAlljson(JSON.toJSONString(ztmap));//总体分析
        epa.setQnumjson(JSON.toJSONString(typelist));//试题栏题型分析
        epa.setComplexityjson(JSON.toJSONString(clist));//难易度分析
        epa.setPointjson(JSON.toJSONString(pointlist));//知识点分析
        epa.setQuespointjson(JSON.toJSONString(quesplist));
        msPaperAnalyzeMapper.insertUseGeneratedKeys(epa);
    }

    //设置知识点
    private void setPaperQuestionPointScore(List<QuestionOrderVo> pointqlist,List<QuestionPointVo> quesplist,List<Map> qpoints,MsPaperQtypeQuestionVo dq,String[] qnstr,List<Integer> mptqids){
        if(N_Utils.isListNotEmpty(qpoints)){
            if(qpoints.size() == 1){
                //试题只有一个知识点
                Map pointmap = qpoints.get(0);
                QuestionOrderVo qpoint = new QuestionOrderVo();
                qpoint.setId(pointmap.get("pointid").toString());
                qpoint.setName(pointmap.get("pointname").toString());
                qpoint.setOrder(dq.getQorder());
                qpoint.setScore(dq.getScore());
                qpoint.setQlevel(dq.getQlevel());
                qpoint.setQuestionid(dq.getQuestionid());
                if(qnstr == null){
                    qpoint.setQns(new String[]{dq.getQn()});
                }else{
                    qpoint.setQns(qnstr);
                }

                if(N_Utils.isListEmpty(mptqids)){
                    qpoint.setMptqids(new Integer[]{dq.getMptqid()});
                }else{
                    Integer[] mptqidstr = mptqids.stream().toArray(Integer[]::new);
                    qpoint.setMptqids(mptqidstr);
                }
                pointqlist.add(qpoint);

                //试题对应知识点
                QuestionPointVo questionPointVo = new QuestionPointVo();
                questionPointVo.setOrder(dq.getQorder());
                questionPointVo.setScore(dq.getScore());
                questionPointVo.setQlevel(dq.getQlevel());
                questionPointVo.setQuestionid(dq.getQuestionid());
                List<Map> pointids = new ArrayList<>();
                Map map = new HashMap();
                map.put("pointid",pointmap.get("pointid").toString());
                map.put("score",dq.getScore());
                pointids.add(map);
                questionPointVo.setPointids(pointids);

                if(N_Utils.isListEmpty(mptqids)){
                    List<Integer> saveids = new ArrayList<>();
                    saveids.add(dq.getMptqid());
                    questionPointVo.setMptqids(saveids);
                    questionPointVo.setMptqid(dq.getMptqid());
                }else{
                    questionPointVo.setMptqids(mptqids);
                    questionPointVo.setMptqid(0);
                }
                quesplist.add(questionPointVo);
            }else{

                //试题有多个知识点
                //计算每个知识点所占分值（平均分配）
                Double[] avgps = N_Utils.getPointAvgScore(qpoints.size(),dq.getScore());
                List<Map> pointids = new ArrayList<>();
                for(int m = 0; m<qpoints.size();m++){
                    Map pointmap = qpoints.get(m);
                    QuestionOrderVo qpoint = new QuestionOrderVo();
                    qpoint.setId(pointmap.get("pointid").toString());
                    qpoint.setName(pointmap.get("pointname").toString());
                    qpoint.setOrder(dq.getQorder());
                    qpoint.setQlevel(dq.getQlevel());
                    qpoint.setQuestionid(dq.getQuestionid());


                    Map quesmap = new HashMap();//试题对应知识点
                    quesmap.put("pointid",pointmap.get("pointid").toString());

                    if(avgps.length == 2 && (m+1) == qpoints.size()){
                        //不能整除分，并且是最后一个知识点
                        qpoint.setScore(avgps[1]);
                        quesmap.put("score",avgps[1]);
                    }else{
                        qpoint.setScore(avgps[0]);
                        quesmap.put("score",avgps[0]);
                    }
                    pointids.add(quesmap);

                    //题号集合
                    if(qnstr == null){
                        qpoint.setQns(new String[]{dq.getQn()});
                    }else{
                        qpoint.setQns(qnstr);
                    }
                    //试卷中试题id集合
                    if(N_Utils.isListEmpty(mptqids)){
                        qpoint.setMptqids(new Integer[]{dq.getMptqid()});
                    }else{
                        Integer[] eptqidstr = mptqids.stream().toArray(Integer[]::new);
                        qpoint.setMptqids(eptqidstr);
                    }
                    pointqlist.add(qpoint);
                }

                //试题对应知识点
                QuestionPointVo questionPointVo = new QuestionPointVo();
                questionPointVo.setOrder(dq.getQorder());
                questionPointVo.setScore(dq.getScore());
                questionPointVo.setQlevel(dq.getQlevel());
                questionPointVo.setQuestionid(dq.getQuestionid());
                questionPointVo.setPointids(pointids);
                if(N_Utils.isListEmpty(mptqids)){
                    List<Integer> saveids = new ArrayList<>();
                    saveids.add(dq.getMptqid());
                    questionPointVo.setMptqids(saveids);
                    questionPointVo.setMptqid(dq.getMptqid());

                }else{
                    questionPointVo.setMptqids(mptqids);
                    questionPointVo.setMptqid(0);
                }
                quesplist.add(questionPointVo);

            }
        } else{
            QuestionPointVo questionPointVo = new QuestionPointVo();
            questionPointVo.setOrder(dq.getQorder());
            questionPointVo.setScore(dq.getScore());
            questionPointVo.setQlevel(dq.getQlevel());
            questionPointVo.setQuestionid(dq.getQuestionid());
            questionPointVo.setPointids(null);
            if(N_Utils.isListEmpty(mptqids)){
                List<Integer> saveids = new ArrayList<>();
                saveids.add(dq.getMptqid());
                questionPointVo.setMptqids(saveids);
                questionPointVo.setMptqid(dq.getMptqid());
            }else{
                questionPointVo.setMptqids(mptqids);
                questionPointVo.setMptqid(0);
            }

            quesplist.add(questionPointVo);
        }
    }

    //设置题型下试题题号及分值
    private void setPaperQuestionQtypeScore(List<QuestionOrderVo> qtypelist,MsPaperQtypeQuestionVo dq,String[] qnstr,Integer[] mptqidstr){
        QuestionOrderVo qtype = new QuestionOrderVo();
        qtype.setId(dq.getQtypeid());
        qtype.setName(dq.getQtypename());
        qtype.setComplexity(dq.getComplexity());
        qtype.setOrder(dq.getQorder());
        qtype.setScore(dq.getScore());
        qtype.setQlevel(dq.getQlevel());
        qtype.setQuestionid(dq.getQuestionid());
        if(qnstr == null){
            qtype.setQns(new String[]{dq.getQn()});
        }else{
            qtype.setQns(qnstr);
        }
        if(mptqidstr == null){
            qtype.setMptqids(new Integer[]{dq.getMptqid()});
        }else{
            qtype.setMptqids(mptqidstr);
        }

        qtypelist.add(qtype);
    }

    //设置试题难易度
    private void setPaperQuestionComplexityScore(List<QuestionOrderVo> complexityList,MsPaperQtypeQuestionVo dq,String[] qnstr,Integer[] mptqidstr){
        QuestionOrderVo qtype = new QuestionOrderVo();
        qtype.setId(dq.getComplexity().toString());//存放试题难易度
        qtype.setOrder(dq.getQorder());
        qtype.setScore(dq.getScore());
        qtype.setQlevel(dq.getQlevel());
        qtype.setQuestionid(dq.getQuestionid());
        if(qnstr == null){
            qtype.setQns(new String[]{dq.getQn()});
        }else{
            qtype.setQns(qnstr);
        }
        if(mptqidstr == null){
            qtype.setMptqids(new Integer[]{dq.getMptqid()});
        }else{
            qtype.setMptqids(mptqidstr);
        }

        complexityList.add(qtype);
    }

    //设置附件
    private void setPaperFiles(MsPaper paper,List<MsPaperFile> pfiles,List<String> imgs) {
        if(paper.getHasfile() == 1){
            int order= 1;
            for(MsPaperFile pf :pfiles){
                pf.setCreateid(paper.getCreateid());
                pf.setCreatetime(paper.getHandletime());
                pf.setMpid(paper.getMpid());
                pf.setFileorder(order);
                order ++;

                if(isImgFormatForsourcepath(pf.getSourcepath()) && !imgs.contains(pf.getSourcepath())){
                    imgs.add(pf.getSourcepath());
                }
            }
        }
    }

    //根据地址判断是否图片
    private boolean isImgFormatForsourcepath(String sourcepath){
        if(sourcepath != null && sourcepath !=""){
            if(sourcepath.contains(".jpg") || sourcepath.contains(".png") || sourcepath.contains(".jpeg")){
                return true;
            }else{
                return false;
            }
        }else{
            return false;
        }

    }

    //保存试卷分析--附件
    private void savePaperAnalyzeForFj(MsPaper paper){
        //保存之前先清除之前的试卷分析
        msPaperAnalyzeMapper.deleteByMpId(paper.getMpid());

        List<MsPaperQtypeVo> fjtypelist = msPaperQtypeMapper.listPaperQtypeQuesitonsForFj(paper.getMpid());
        List<MsPaperQtypeQuestion> queslist = new ArrayList<>();
        for(MsPaperQtypeVo fj : fjtypelist){
            List<MsPaperQtypeQuestion> qlist = fj.getQuestionfjs();
            for(MsPaperQtypeQuestion q : qlist){
                queslist.add(q);
            }
        }

        //总体分析（主观题、客观题）
        //处理题型1单选题2多选题3主观题4判断对错5判断√×6判断TF11综合题12听力13填空题
        int znum = 0;
        double zscore = 0;
        int knum = 0;
        double kscore = 0;
        for(MsPaperQtypeQuestion q : queslist){
            if(N_Utils.isObjectiveQuestion(q.getCtype())){
                knum ++;
                kscore = kscore + q.getQscore();
            }else{
                znum ++;
                zscore = zscore + q.getQscore();
            }
        }

        double znumrate = N_Utils.getIntegerDivideAndMulitiply(znum, paper.getPnum());
        double knumrate = N_Utils.getDoubleReduce(100.0, znumrate);

        double zscorerate = N_Utils.getDoubleDivideAndMulitiply(zscore, paper.getPscore());
        double kscorerate = N_Utils.getDoubleReduce(100.0,zscorerate);

        List<Map> ranglist = new ArrayList<>();
        Map zgmap = new TreeMap();
        zgmap.put("qtname","主观题");
        zgmap.put("score",zscore);
        zgmap.put("num",znum);
        zgmap.put("srate",zscorerate);
        zgmap.put("nrate",znumrate);
        ranglist.add(zgmap);
        Map kgmap = new TreeMap();
        kgmap.put("qtname","客观题");
        kgmap.put("score",kscore);
        kgmap.put("num",knum);
        kgmap.put("srate",kscorerate);
        kgmap.put("nrate",knumrate);
        ranglist.add(kgmap);

        Map ztmap = new TreeMap<>();
        ztmap.put("pnum", paper.getPnum());
        ztmap.put("pscore",paper.getPscore());
        ztmap.put("ranglist",ranglist);

        //题型分布(按照名称进行分组)
        fjtypelist.stream().sorted(Comparator.comparing(MsPaperQtypeVo::getMptorder)).collect(Collectors.toList());
        Map<String,List<MsPaperQtypeVo>> qtypemap =  fjtypelist.stream().collect(Collectors.groupingBy(MsPaperQtypeVo:: getMptname, Collectors.toList()));
        List<Map> qtlist = new ArrayList<>();
        for(Map.Entry<String, List<MsPaperQtypeVo>> entry : qtypemap.entrySet()){
            List<MsPaperQtypeVo> aq = entry.getValue();
            int num = aq.stream().mapToInt(MsPaperQtypeVo::getMptnum).sum();
            double tscore = aq.stream().collect(Collectors.summingDouble(MsPaperQtypeVo:: getMptscore));
            tscore = N_Utils.formatDouble(tscore,1);
            double storerate = N_Utils.getDoubleDivideAndMulitiply(tscore, paper.getPscore());
            //获取小题题号

            List<Integer> orderlist = new ArrayList<>();
            List<Integer> mptqidlist = new ArrayList<>();
            List<String> qnlist = new ArrayList<>();
            List<Double> scorelist = new ArrayList<>();
            List<String> qnamelist = new ArrayList<>();
            for(MsPaperQtypeVo t : aq){
                List<MsPaperQtypeQuestion> qlist = t.getQuestionfjs();
                for(MsPaperQtypeQuestion q : qlist){
                    orderlist.add(q.getQorder());
                    mptqidlist.add(q.getId());
                    qnlist.add(q.getQn());
                    scorelist.add(q.getQscore());
                    qnamelist.add(q.getQtypename());
                }
            }

            Map anmap = new TreeMap();//题型对应试题的id和基础信息
            anmap.put("orders",orderlist);
            anmap.put("mptqids",mptqidlist);
            anmap.put("qns",qnlist);
            anmap.put("scores",scorelist);
            anmap.put("qtypenames",qnamelist);

            Map tmap = new TreeMap();
            tmap.put("qtname",entry.getKey());
            tmap.put("qtid",aq.get(0).getMptid());
            tmap.put("num",num);
            tmap.put("score",tscore);
            tmap.put("srate",storerate);
            tmap.put("ques",anmap);
            qtlist.add(tmap);
        }

        MsPaperAnalyze epa = new MsPaperAnalyze();
        epa.setMpid(paper.getMpid());
        epa.setAlljson(JSON.toJSONString(ztmap));
        epa.setQnumjson(JSON.toJSONString(qtlist));
        msPaperAnalyzeMapper.insertUseGeneratedKeys(epa);
    }


    /*
     * @Description 试卷答案-客观题
     * @Date 2024/10/17 15:38:44
     * @Author WN
     * @Param [qtype]
     * @Return com.xhkjedu.vo.ResultVo
     **/
    public ResultVo answerObjective(MsPaperQtype qtype){
        Integer mpid = qtype.getMpid();
        List<MsPaperQtypeQuestion> questions = qtype.getQuestions();
        for(MsPaperQtypeQuestion question : questions) {
            if(N_Utils.isTrueInteger(question.getMptqid()) || N_Utils.isEmpty(question.getQanswer())){
                return new ResultVo(1,"请设置对应试题答案");
            }
        }

        //获取试题答案改变的试题
        List<MsPaperQtypeQuestion> changelist = listChangeQuestionAnswer(questions,mpid);

        //获取考试状态
        int examstate = 0;


        saveQuestionAnswerObjective(mpid,changelist);

        //判断考试状态是开始，已提交学生重新批阅

        return new ResultVo(0,"保存成功");
    }
    //获取答案改变的试题
    private List<MsPaperQtypeQuestion> listChangeQuestionAnswer(List<MsPaperQtypeQuestion> questions,Integer mpid){
        List<MsPaperQtypeQuestion> changelist = new ArrayList<>();
        //获取试卷中原试题答案
        List<MsPaperQtypeQuestion> yqlist = msPaperQtypeQuestionMapper.listQuestionsByMpid(mpid);
        for(MsPaperQtypeQuestion tq : questions){
            if(N_Utils.isTrueInteger(tq.getMptqid()) && N_Utils.isNotEmpty(tq.getQanswer())){
                MsPaperQtypeQuestion yq = yqlist.stream().filter(y -> y.getMptqid().equals(tq.getMptqid())).findFirst().orElse(null);

                if(N_Utils.isObjectiveQuestion(yq.getCtype())){
                    if(!tq.getQanswer().equalsIgnoreCase(yq.getQanswer()) || !tq.getLoseoption().equals(yq.getLoseoption()) || !tq.getScoreset().equals(yq.getScoreset())){
                        //说明新设置的答案和原设置的答案不一致，则为变动的
                        tq.setCtype(yq.getCtype());
                        tq.setQscore(yq.getQscore());
                        changelist.add(tq);
                    }
                }
            }
        }
        return changelist;
    }

    //试卷保存客观题答案-保存答案
    @Transactional(rollbackFor = Exception.class)
    public void saveQuestionAnswerObjective(Integer mpid,List<MsPaperQtypeQuestion> questions){
        //更新试卷设置答案状态
        msPaperMapper.updatePaperAnswer(1,mpid);
        //保存试卷答案
        msPaperQtypeQuestionMapper.updateBatchQuestionAnswer(questions);
        //保存试卷模版题块中试题答案

        //如果考试一开始，标记需要批改的学生试题
    }

    /*
     * @Description 试卷答案-主观题
     * @Date 2024/10/17 15:40:47
     * @Author WN
     * @Param [qtype]
     * @Return com.xhkjedu.vo.ResultVo
     **/
    @Transactional(rollbackFor = Exception.class)
    public void answerSubjective(MsPaper paper){
        Integer mpid = paper.getMpid();
        List<MsPaperFile> pfiles = paper.getPfiles();
        //删除已保存主观题答案图片
        msPaperFileMapper.deleteByMpidAndType(mpid,2);
        Integer timestamp = N_Utils.getSecondTimestamp();
        for (MsPaperFile pf : pfiles) {
            pf.setMpid(mpid);
            pf.setCreatetime(timestamp);
            pf.setFiletype(2);
        }
        msPaperFileMapper.insertList(pfiles);
    }

    /*
     * @Description 试卷题干文件
     * @Date 2024/10/17 16:00:37
     * @Author WN
     * @Param [paper]
     * @Return com.xhkjedu.vo.ResultVo
     **/
    @Transactional(rollbackFor = Exception.class)
    public void savePaperFiles(MsPaper paper){
        Integer mpid = paper.getMpid();
        List<MsPaperFile> pfiles = paper.getPfiles();
        //删除已保存主观题答案图片
        msPaperFileMapper.deleteByMpidAndType(mpid,1);
        Integer timestamp = N_Utils.getSecondTimestamp();
        for (MsPaperFile pf : pfiles) {
            pf.setMpid(mpid);
            pf.setCreatetime(timestamp);
            pf.setFiletype(1);
        }
        msPaperFileMapper.insertList(pfiles);
    }

    /*
     * @Description 保存试题知识点和难易度
     * @Date 2024/10/17 17:07:28
     * @Author WN
     * @Param [paper]
     * @Return void
     **/
    @Transactional(rollbackFor = Exception.class)
    public void saveQuestionPoints(MsPaper paper){
        List<MsPaperQtypeQuestion> questions = paper.getQuestions();
        List<MsPaperQtypeQuestionPoint> savePoints = new ArrayList<>();
        Integer mpid = paper.getMpid();
        Integer createid = paper.getCreateid();
        Integer timestamp = N_Utils.getSecondTimestamp();
        for(MsPaperQtypeQuestion question : questions){
            List<MsPaperQtypeQuestionPoint> points = question.getPoints();
            for(MsPaperQtypeQuestionPoint point : points){
                point.setCreateid(createid);
                point.setCreatetime(timestamp);
                point.setMptqid(question.getMptqid());
                point.setMpid(mpid);
                savePoints.add(point);
            }
        }
        //保存难易度
        msPaperQtypeQuestionMapper.updateBatchQuestionComplexity(questions);

        //保存试题知识点
        msPaperQtypeQuestionPointMapper.insertList(savePoints);

        //保存完知识点难易度后，更新试卷分析
        MsPaper apaper = msPaperMapper.selectByPrimaryKey(mpid);
        savePaperAnalyzeForFj(apaper);

        //生成考试报告中知识点难易度相关分析
    }

    /*
     * @Description 获取试卷详细信息
     * @Date 2024/10/18 9:29:37
     * @Author WN
     * @Param [msid]
     * @Return java.util.Map<java.lang.String,java.lang.Object>
     **/
    public Map<String,Object> getPaperDetail(Integer msid){
        //获取试卷详情
        Map<String,Object> paper = msPaperMapper.getPaperInfoByMsid(msid);
        if(N_Utils.isEmpty((paper.get("ptype"))) || paper.get("ptype").equals("0")){
            return paper;
        }
        int ptype = Integer.parseInt(paper.get("ptype").toString());
        Integer mpid = Integer.parseInt(paper.get("mpid").toString());
        List<MsPaperQtypeVo> qtypelist = new ArrayList<>();
        //获取题型下试题
        if(ptype == 1){
            //题库
            qtypelist = msPaperQtypeMapper.listPaperQtypeQuestions(mpid);
        }else{
            //附件
            qtypelist = msPaperQtypeMapper.listPaperQtypeQuesitonsForFj(mpid);
            //试卷文件
            List<MsPaperFile> pfiles = msPaperFileMapper.listFilesbyMpid(mpid);
            paper.put("pfiles",pfiles);
        }
        paper.put("qtypes",qtypelist);

        //试卷分析
        MsPaperAnalyze analyze = msPaperAnalyzeMapper.getObjByMpId(mpid);
        paper.put("analyze",analyze);

        return paper;

    }


    /*
     * @Description 获取试卷中所有主观题（题型-试题）
     * @Date 2024/10/21 13:50:54
     * @Author WN
     * @Param [mpid]
     * @Return java.util.List<java.util.Map<java.lang.String,java.lang.Object>>
     **/
    public List<Map<String,Object>> listPaperSubjectiveQuestions(Integer mpid){
        MsPaper paper = msPaperMapper.getPaperBaseByMsid(mpid);
        int[] subjectivelist = N_Utils.subjectiveList;
        // List<Map<String,Object>> rtnList = new ArrayList<>();
        // if(paper.getPtype() == 1){
        //
        // }else{
        //     rtnList = msPaperQtypeMapper.listSubjectiveQuestionsFj(mpid,subjectivelist);
        // }

        return msPaperQtypeMapper.listSubjectiveQuestionsFj(mpid,subjectivelist);
    }
}
