package com.xhkjedu.smarking.service.paper;

import com.xhkjedu.smarking.mapper.exam.MsSubjectMapper;
import com.xhkjedu.smarking.mapper.paper.*;
import com.xhkjedu.smarking.model.paper.*;
import com.xhkjedu.smarking.vo.paper.MsPaperParams;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description 阅卷扫描模板表 服务实现类
 * @Author auto
 * @Date 2024-10-15
 */
@Service
public class MsTemplateService {
    @Resource
    private MsTemplateMapper msTemplateMapper;
    @Resource
    private MsTemplateBlockMapper msTemplateBlockMapper;
    @Resource
    private MsTemplateBlockQuestionMapper msTemplateBlockQuestionMapper;
    @Resource
    private MsPaperQtypeQuestionMapper msPaperQtypeQuestionMapper;
    @Resource
    private MsPaperBlockMapper msPaperBlockMapper;
    @Resource
    private MsPaperBlockQuestionMapper msPaperBlockQuestionMapper;
    @Resource
    private MsSubjectMapper msSubjectMapper;

    /*
     * @Description 试卷模板列表
     * @Date 2024/10/18 14:44:33
     * @Author WN
     * @Param [msid]
     * @Return java.util.List<java.util.Map<java.lang.String,java.lang.Object>>
     **/
    public List<Map<String, Object>> listTemplatesByMsid(Integer msid){
        return msTemplateMapper.listTemplatesByMsid(msid);
    }

    /**
     * @Description 新增
     * @Param [msTemplate]
     * @Author auto
     * @Date 2024-10-15
     * @return void
     **/
    @Transactional(rollbackFor = Exception.class)
    public MsTemplate save(MsTemplate msTemplate){
        msTemplate.setCreatetime(N_Utils.getSecondTimestamp());
        msTemplate.setCurrstep(1);
        msTemplate.setTstate(0);
        msTemplate.setTbstate(0);
        msTemplateMapper.insertUseGeneratedKeys(msTemplate);
        int mtid = msTemplate.getId();
        msTemplate.setMtid(mtid);

        //更新科目模板状态
        msSubjectMapper.updatePtstateByMpid(msTemplate.getMpid(),1);
        return msTemplate;
    }

    /*
     * @Description 设置模版
     * @Date 2024/10/21 10:47:00
     * @Author WN
     * @Param [msTemplate]
     * @Return com.xhkjedu.smarking.model.paper.MsTemplate
     **/
    @Transactional(rollbackFor = Exception.class)
    public MsTemplate updateTemplate(MsTemplate msTemplate){
        msTemplateMapper.updateTemplateForCurrstep(msTemplate);
        //如果当前保存为6，则需要保存客观题信息
        if(msTemplate.getCurrstep() == 6){
            //模版题块信息
            List<MsTemplateBlock> tblocks = msTemplate.getTblocks();
            savePaperBlockAndTemplate(tblocks,msTemplate);
        }

        if(msTemplate.getTstate()!=null && msTemplate.getTstate() == 1){
            Integer mpid = msTemplate.getMpid();
            //代表提交,获取试卷对应所有的模块状态，如果均是已提交，则更改科目表中状态
            Integer uncommit = msTemplateMapper.getUnCommitTemplateCount(mpid);
            if(uncommit == 0){
                msSubjectMapper.updatePtstateByMpid(mpid,2);
            }
        }
        return msTemplate;
    }

    //保存客观题题块和模版信息
    private void savePaperBlockAndTemplate(List<MsTemplateBlock> tblocks,MsTemplate msTemplate){
        Integer mpid = msTemplate.getMpid();
        Integer timestamp = N_Utils.getSecondTimestamp();
        Integer createid = msTemplate.getCreateid();

        //保存之前先清除已保存的信息
        msTemplateBlockMapper.deleteByMtidAndMtype(msTemplate.getMtid(),1);
        msPaperBlockMapper.deleteObjByMpidAndBlocktype(mpid,1);

        //客观题题块试题信息
        List<MsPaperBlockQuestion> savePbquestions = new ArrayList<>();
        //客观题模块试题信息
        List<MsTemplateBlockQuestion> saveTbquestions = new ArrayList<>();
        //获取试卷中所有试题信息
        List<MsPaperQtypeQuestion> questions = msPaperQtypeQuestionMapper.listQuestionsByMpid(mpid);
        int i = 1;
        for(MsTemplateBlock tblock : tblocks){
            //获取客观题单题
            List<MsTemplateBlockQuestion> tbquestions = tblock.getTbquestions();
            //获取客观题题号
            List<String> tbqns = tbquestions.stream().map(MsTemplateBlockQuestion::getQn).collect(Collectors.toList());
            //包含的试题信息
            List<MsPaperQtypeQuestion> selQuestions = questions.stream().filter(q -> tbqns.contains(q.getQn())).collect(Collectors.toList());
            if(selQuestions == null || selQuestions.size() == 0 || selQuestions.size() != tbquestions.size()){
                throw new RuntimeException("试题信息和试卷不匹配");
            }

            Double tbqScores = selQuestions.stream().mapToDouble(MsPaperQtypeQuestion::getQscore).sum();//题块下试题总分值

            //保存试卷题块信息
            MsPaperBlock paperBlock = new MsPaperBlock();
            paperBlock.setMpid(mpid);
            paperBlock.setBlockname("客观题题块" + (i++));
            paperBlock.setBlocktype(1);
            paperBlock.setBlockorder(tblock.getBlockorder());
            paperBlock.setBqnum(tbquestions.size());
            paperBlock.setBqscore(tbqScores);
            paperBlock.setCreateid(createid);
            paperBlock.setCreatetime(timestamp);
            msPaperBlockMapper.insertUseGeneratedKeys(paperBlock);
            Integer mblockid = paperBlock.getId();//题块ID

            //保存模块题块信息
            tblock.setBlockname(paperBlock.getBlockname());
            tblock.setMttype(1);
            tblock.setMblockid(mblockid);
            tblock.setCreateid(createid);
            tblock.setCreatetime(timestamp);
            msTemplateBlockMapper.insertUseGeneratedKeys(tblock);
            Integer mtbid = tblock.getId();

            for(MsTemplateBlockQuestion tbquestion : tbquestions) {
                tbquestion.setMtbid(mtbid);
                saveTbquestions.add(tbquestion);

                MsPaperQtypeQuestion q = selQuestions.stream().filter(q1 -> q1.getQn().equals(tbquestion.getQn())).findFirst().get();

                MsPaperBlockQuestion pbquestion = new MsPaperBlockQuestion();
                pbquestion.setMblockid(mblockid);
                pbquestion.setMpid(mpid);
                pbquestion.setMbqtype(1);
                pbquestion.setMptqid(q.getMptqid());
                pbquestion.setBqn(tbquestion.getQn());
                pbquestion.setBqscore(q.getQscore());
                pbquestion.setBqorder(tbquestion.getBqorder());
                pbquestion.setCreateid(createid);
                pbquestion.setCreatetime(timestamp);
                savePbquestions.add(pbquestion);
            }
        }

        msPaperBlockQuestionMapper.insertList(savePbquestions);
        msTemplateBlockQuestionMapper.insertList(saveTbquestions);
    }


    /*
     * @Description 保存模板主观题框选
     * @Date 2024/10/21 17:41:42
     * @Author WN
     * @Param [params]
     * @Return void
     **/
    @Transactional(rollbackFor = Exception.class)
    public void saveSubjectiveQuestion(MsPaperParams params){
        Integer mtid = params.getMtid();
        Integer mpid = params.getMpid();
        Integer createid = params.getCreateid();
        Integer timestamp = N_Utils.getSecondTimestamp();
        List<MsTemplateBlock> templateBlocks = params.getTblocks();
        //删除模块中题块定位数据
        msTemplateBlockMapper.deleteByMtidAndMtype(mtid,2);

        for (MsTemplateBlock templateBlock : templateBlocks){
            templateBlock.setMtid(mtid);
            templateBlock.setMttype(2);
            templateBlock.setCreateid(createid);
            templateBlock.setCreatetime(timestamp);

            msTemplateBlockMapper.insertList(templateBlocks);
        }
        //更改模版题块框选状态
        msTemplateMapper.updateTemplateTbstate(mtid,1);
        //获取试卷中所有模版题块框选状态
        int tbnum = msTemplateMapper.getUnSelTemplateCount(mpid);
        Integer pbstate = 0;
        if(tbnum == 0){
            pbstate = 2;
        }else{
            pbstate = 1;

        }
        //更改科目题块框选状态
        msSubjectMapper.updatePbstateByMpid(mpid,pbstate);
    }

    /*
     * @Description 删除模板
     * @Date 2024/10/21 17:49:46
     * @Author WN
     * @Param [mtid]
     * @Return void
     **/
    public void deleteTemplate(Integer mtid){
        msTemplateMapper.deleteByPrimaryKey(mtid);
    }
}
